#!/usr/bin/env python3

import time
import curses
import math


class Plant:
    def __init__(self):
        self.height = 0
        self.water = 100
        self.food = 100
        self.pot = 5
        self.timelastgrown = time.time()
        self.health = 50
        self.plantwidth = 0
        self.screen = None
        return

    def grow(self):
        if time.time() > self.timelastgrown + 1 and self.health > 0:
            self.timelastgrown = time.time()
            self.consume()
            self.heal()
            if self.food > 0 and self.water > 0 and self.height < self.pot:
                self.height += 1
                if self.plantwidth < self.pot // 2 - 1:
                    self.plantwidth += 1
        if self.health <= 0:
            self.screen.update("Your plant died.")
            return
        self.screen.update()
        return

    def getattr(self):
        attrs = {"Health": self.health, "Water": self.water, "Food": self.food}
        return attrs

    def add_water(self):
        self.water += 35
        return

    def add_food(self):
        self.food += 35
        return

    def consume(self):
        self.water -= self.height
        if self.water < 0:
            self.water = 0
        self.food -= math.ceil(self.height / 3)
        if self.food < 0:
            self.food = 0
        return

    def heal(self):
        self.health += 10 - math.ceil(math.pow(self.water - 75, 2) / 281.25)
        self.health += 10 - math.ceil(math.pow(self.food - 75, 2) / 281.25)
        if self.health > 100:
            self.health = 100
        if self.health < 0:
            self.health = 0
        return

    def repot(self):
        self.pot += 4
        self.consume()
        return


class Screen:
    def __init__(self, stdscr, plant):
        self.stdscr = stdscr
        self.plant = plant
        self.plant.screen = self
        self.statuspos = curses.COLS - 7
        curses.curs_set(False)
        curses.noecho()
        curses.halfdelay(1)
        self.stdscr.keypad(True)
        curses.init_pair(1, curses.COLOR_RED, curses.COLOR_BLACK)
        self.potcolour = curses.color_pair(1)
        curses.init_pair(2, curses.COLOR_GREEN, curses.COLOR_BLACK)
        self.plantokcolour = curses.color_pair(2)
        curses.init_pair(3, curses.COLOR_YELLOW, curses.COLOR_BLACK)
        self.plantbadcolour = curses.color_pair(3)
        self.stdscr.clear()
        self.stdscr.addstr(1, 1, "[W] Add Water")
        self.stdscr.addstr(3, 1, "[F] Add Food")
        self.stdscr.addstr(5, 1, "[P] Re-pot")
        self.stdscr.addstr(curses.LINES - 4, 1, "[R] Restart")
        self.stdscr.addstr(curses.LINES - 3, 1, "[Q] Quit")
        return

    def update(self, msg_override=None):
        self.drawplant()
        self.writeinfo()
        self.writemsg(msg_override)
        self.stdscr.refresh()

    def writeinfo(self):
        attrs = self.plant.getattr()
        maxlen = 0
        for k in attrs.keys():
            if len(k) > maxlen:
                maxlen = len(k)
        line = 1
        for k, v in attrs.items():
            self.stdscr.addstr(line, self.statuspos - maxlen, "{0}:{1}{2: 4}".format(k, " " * (maxlen - len(k)), v))
            line += 2
        return

    def drawplant(self):
        line = curses.LINES - 2
        if self.plant.pot >= curses.COLS:
            self.stdscr.addstr(line, 0, "#" * curses.COLS, self.potcolour)
        else:
            self.stdscr.addstr(line, 0, "{0}{1}{0}".format(" " * math.floor((curses.COLS - self.plant.pot) / 2),
                                                           "#" * self.plant.pot), self.potcolour)
        plantloc = math.floor(curses.COLS / 2)
        plantcolour = self.plantokcolour
        if self.plant.health < 50:
            plantcolour = self.plantbadcolour
        plantwidth = self.plant.plantwidth
        for i in range(self.plant.height):
            line -= 1
            if line >= 0:
                if line % 2 == 0:
                    self.stdscr.addstr(line, plantloc, "|", plantcolour)
                else:
                    leaves = "/" + "-" * plantwidth + "|" + "-" * plantwidth + "\\"
                    self.stdscr.addstr(line, plantloc - plantwidth - 1, leaves, plantcolour)
                    plantwidth -= 1
        for i in range(line):
            line -= 1
            if line >= 0:
                self.stdscr.addstr(line, plantloc, " ")
        return

    def writemsg(self, override=None):
        msg = " "
        if override is not None:
            msg = override
        elif self.plant.water < 50:
            msg = "Your plant needs water!"
        elif self.plant.food < 50:
            msg = "Your plant needs food!"
        elif self.plant.water > 150:
            msg = "Don't over-water!"
        elif self.plant.food > 150:
            msg = "Don't over-feed!"
        elif self.plant.height >= self.plant.pot - 2:
            msg = "You need a bigger pot!"
        self.stdscr.addstr(curses.LINES - 3, curses.COLS - 24, msg.rjust(23))
        return


def main(stdscr):
    plant = Plant()
    screen = Screen(stdscr, plant)
    while True:
        c = stdscr.getch()
        if c == ord('w'):
            plant.add_water()
        if c == ord('f'):
            plant.add_food()
        if c == ord('p'):
            plant.repot()
        if c == ord('q'):
            return
        if c == ord('r'):
            plant = Plant()
            screen = Screen(stdscr, plant)
        plant.grow()


if __name__ == "__main__":
    curses.wrapper(main)
